'use strict';
var gulp = require('gulp');
var gutil = require('gulp-util');
var del = require('del');
var uglify = require('gulp-uglify');
var gulpif = require('gulp-if');
var exec = require('child_process').exec;
var concat = require('gulp-concat');
var runSequence = require('run-sequence');

var partialify = require('partialify');
var notify = require('gulp-notify');
var buffer = require('vinyl-buffer');
var argv = require('yargs').argv;
var sass = require('gulp-sass');
var postcss = require('gulp-postcss');
var autoprefixer = require('autoprefixer-core');
var sourcemaps = require('gulp-sourcemaps');
var browserSync = require('browser-sync');
var watchify = require('watchify');
var browserify = require('browserify');
var source = require('vinyl-source-stream');
var imagemin = require('gulp-imagemin');
var jshint = require('gulp-jshint');
var stylish = require('jshint-stylish');

// gulp build --production
var production = !!argv.production;

var build = argv._.length ? argv._[0] === 'build' : false;
var watch = argv._.length ? argv._[0] === 'watch' : true;

var handleError = function(task) {
  return function(err) {
    gutil.log(gutil.colors.bgRed(task + ' error:'), gutil.colors.red(err));
  };
};
var tasks = {
  clean: function() {
    return del(['www/']);
  },
  templates: function() {
    return gulp.src('./src/index.html')
      .pipe(gulp.dest('www/'));
  },
  sass: function() {
    return gulp.src([
      './src/**/*.scss'
    ])
    .pipe(gulpif(!production, sourcemaps.init()))
    .pipe(sass({
      sourceComments: !production,
      outputStyle: production ? 'compressed' : 'nested'
    }))
    .on('error', handleError('SASS'))
    .pipe(gulpif(!production, sourcemaps.write({
      'includeContent': false,
      'sourceRoot': '.'
    })))
    .pipe(gulpif(!production, sourcemaps.init({
      'loadMaps': true
    })))
    .pipe(postcss([autoprefixer({browsers: ['last 2 versions']})]))
    .pipe(sourcemaps.write({
      'includeContent': true
    }))
    .pipe(concat('style.css'))
    .pipe(gulp.dest('www/css'));
  },
  browserify: function() {
    var bundler = browserify('./src/js/app.js', {
      debug: !production,
      cache: {},
      extensions: ['.js', '.html']
    });
    var build = argv._.length ? argv._[0] === 'build' : false;
    if (watch) {
      bundler = watchify(bundler);
    }
    var rebundle = function() {
      return bundler.transform(partialify)
        .bundle()
        .on('error', handleError('Browserify'))
        .pipe(source('./bundle.js'))
        .pipe(gulpif(production, buffer()))
        // angular breaks without mangle: false
        .pipe(gulpif(production, uglify({ mangle: false })))
        .pipe(gulp.dest('www/scripts'));
    };
    bundler.on('update', rebundle);
    return rebundle();
  },
  lintjs: function() {
    return gulp.src([
        'gulpfile.js',
        './src/**/*.js'
      ])
      .pipe(jshint())
      .pipe(jshint.reporter(stylish))
  },
  optimize: function() {
    return gulp.src('./src/assets/**/*.{gif,jpg,png,svg}')
      .pipe(imagemin({
        progressive: true,
        svgoPlugins: [{removeViewBox: false}],
        optimizationLevel: production ? 3 : 1
      }))
      .pipe(gulp.dest('./www/assets'));
  }
};

gulp.task('browser-sync', function() {
  browserSync({
    server: {
      baseDir: "./www"
    },
    port: process.env.PORT || 3000
  });
});

gulp.task('reload-sass', ['sass'], function(){
  browserSync.reload();
});
gulp.task('reload-js', ['browserify'], function(){
  browserSync.reload();
});
gulp.task('reload-templates', ['browserify'], function(){
  browserSync.reload();
});

gulp.task('images:fonts', function() {
  gulp.src('./src/fonts/*.*')
    .pipe(gulp.dest('./www/fonts'));

  return gulp.src('./vendor/ionic/release/fonts/*.*')
    .pipe(gulp.dest('./www/fonts'));
});

gulp.task('clean', tasks.clean);
var req = build ? ['clean'] : [];
gulp.task('templates', req, tasks.templates);
gulp.task('sass', req, tasks.sass);
gulp.task('browserify', req, tasks.browserify);
gulp.task('lint:js', tasks.lintjs);
gulp.task('optimize', tasks.optimize);

gulp.task('watch', function() {

  gulp.watch('./src/**/*.scss', ['reload-sass']);
  gulp.watch('./src/**/*.js', ['lint:js', 'reload-js']);
  gulp.watch('./src/**/*.html', ['templates', 'browserify']);
});

gulp.task('build', function() {
  runSequence(
    'clean',
    ['templates', 'images:fonts', 'sass', 'browserify', 'optimize']
  )
});

gulp.task('serve', function() {
  runSequence(
    'clean',
    ['templates', 'images:fonts', 'sass', 'browserify', 'optimize'],
    'watch',
    'browser-sync');
});

// gulp (watch) : for development and livereload
// gulp build : for a one off development build
// gulp build --production : for a minified production build