var tables = require('../tables')
var api = require('./index')
var database = require('../database')
var Session = require('../session')
var statusProvider = require('../lib/status/provider')

var squel = require('squel')
var Promise = require('bluebird')

var DRAFT_STATUS = statusProvider.draft
var CLOSED_STATUS = statusProvider.closed
var EDITED_STATUS = statusProvider.edited

var apiSyncer = function() {}

apiSyncer._getApiFields = function(fields) {
  var self = this
  return Object.keys(fields)
    .filter(function(fieldName) {
      return !fields[fieldName].local
    });
}

apiSyncer._getLines = function(table, entities) {
  var getLinePromises = []
  entities.forEach(function(entity) {
    var lineTable = tables[table.line.tableName]
    var query = squel.select()
      .from(lineTable.name)
      .where(table.line.foreignKey + ' = ?', entity.id)

    var promise = database.executeSql(query.toString())
      .then(function(lines) {
        entity.lines = lines
        return Promise.resolve(entity)
      })

    getLinePromises.push(promise)
  })

  return Promise.all(getLinePromises)
}

apiSyncer._getDraft = function(table, filters) {
  if (!filters) filters = {}
  var self = this
  var query = squel.select()
    .from(table.name)
    .where('status = ?', DRAFT_STATUS)

  Object.keys(filters).forEach(function(field) {
    var value = filters[field]
    query.where(field + ' = ?', value)
  })

  return database.executeSql(query.toString())
    .then(function(entities) {
      if (!table.line) return Promise.resolve(entities)

      return self._getLines(table, entities)
    })
}

apiSyncer._getEdited = function(table, filters) {
  if (!filters) filters = {}
  var self = this
  var query = squel.select()
    .from(table.name)
    .where('status = ?', EDITED_STATUS)

  Object.keys(filters).forEach(function(field) {
    var value = filters[field]
    query.where(field + ' = ?', value)
  })

  return database.executeSql(query.toString())
}

apiSyncer._mapRow = function(apiFields, row) {
  return apiFields.reduce(function(acc, fieldName) {
    acc[fieldName] = row[fieldName]
    return acc
  }, {})
}

apiSyncer._getApiRows = function(table) {
  var apiFields = this._getApiFields(table.fields)
  var apiKeys = this._getApiFields(table.keys)

  return apiKeys.concat(apiFields)
}

apiSyncer._mapRows = function(table, rows) {
  var self = this
  var mappedRows = []
  rows.forEach(function(row) {
    var apiRows = self._getApiRows(table)
    apiRows.push('lines')
    mappedRows.push(self._mapRow(apiRows, row))
  })

  return mappedRows
}

apiSyncer._closeDrafts = function(table, filters) {
  if (!filters) filters = {}
  var query = squel.update()
    .table(table.name)
    .set('status', CLOSED_STATUS)

  Object.keys(filters).forEach(function(fieldName) {
    var value = filters[fieldName]
    query.where(fieldName + ' = ?', value)
  })

  return database.executeSql(query.toString())
}

apiSyncer.syncTable = function(table, filters) {
  var payload = []
  var self = this

  return this._getDraft(table, filters)
  .then(function(rows){
    return self._mapRows(table, rows)
  })
  .then(function(payload) {
    if (!payload.length) return Promise.resolve(true)
    return api.post(table.resource, payload)
  })
  .then(function() {
    return self._getEdited(table, filters)
  })
  .then(function(rows){
    return self._mapRows(table, rows)
  })
  .then(function(payload) {
    if (!payload.length) return Promise.resolve(true)
    return api.put(table.resource, payload)
  })
  .then(function() {
    return self._closeDrafts(table, filters)
  })
  .catch(function(error) {
    console.error(error)
    return Promise.reject(error)
  })
 }

/**
 For every table with an API resource url, download data and save in tmp database
**/
apiSyncer.getResources = function() {
  var self = this
  var promises = []

  Object.keys(tables)
  .filter(function(tableName) { return tables[tableName].resource })
  .map(function(tableName) { return tables[tableName] })
  .forEach(function(table) { promises.push(self._phoenix(table)) })

  return Promise.all(promises)
}

/**
  Removes al rows and reloads from API
**/
apiSyncer._phoenix = function(table) {
  var self = this
  var promises = []

  return database.delete(table, {})
    .then(function() { return api.get(table.resource)})
    .then(function(response) { return self._saveData(table, response)})
}

apiSyncer._getTable = function(table) {
  var self = this

  return api.get(table.resource)
    .then(function(response) {
      return self._saveData(table, response)
    })
}

apiSyncer._saveData = function(table, response) {
  if (response.length === 0) return Promise.resolve(true)
  return database.create(table, response)
}

module.exports = apiSyncer
