var Promise = require('bluebird')
var squel = require('squel')
var tables = require('../tables')

var DEBUG = true

var CONFIG = {
  location: 'default',
  name: 'app-db',
  size: 5*1024*1024,  // 5MB
  version: '1.0'
}

var singletonInstance = null

var Database = function() {
  if (singletonInstance) {
    return singletonInstance
  }

  this.db = openDatabase(CONFIG.name, CONFIG.version, CONFIG.name, CONFIG.size)

  this._createTables()

  singletonInstance = this
}

Database.prototype.createTable = function(table) {
  var statement = 'CREATE TABLE IF NOT EXISTS ' + table.name + ' ('

  var primaryKeys = Object.keys(table.keys).map(function(primaryKey) {
    var dataType = table.keys[primaryKey]
    return primaryKey + ' ' + dataType + ' PRIMARY KEY'
  })
  var fields = Object.keys(table.fields).map(function(field) {
    var dataType = typeof table.fields[field] === 'object' ? table.fields[field].type : table.fields[field]
    return field + ' ' + dataType
  })

  statement += primaryKeys.join(', ') + ', ' + fields.join(', ') + ')'

  return this.executeSql(statement)
}

Database.prototype._createTables = function() {
  var self = this
  Object.keys(tables).forEach(function(key) {
    return self.createTable(tables[key])
  })
}

Database.prototype._addFilters = function(query, filters) {
  if (!filters) return
  Object.keys(filters).forEach(function(fieldName) {
    var operation = filters[fieldName].operation ?
      filters[fieldName].operation : '='

    query.where(fieldName + ' ' + operation + ' ?', filters[fieldName])
  })
}

Database.prototype.create = function(table, data) {
  var query = squel.insert()
    .into(table.name)

  var fn = Array.isArray(data) ? 'setFieldsRows' : 'setFields'
  query[fn](data)

  return this.executeSql(query.toString())
}

Database.prototype.read = function(table, fields, filters) {
  var query = squel.select()
    .from(table.name)
    .fields(fields)

  this._addFilters(query, filters)

  return this.executeSql(query.toString())
}

Database.prototype.update = function(table, fields, filters) {
  var query = squel.update()
    .table(table.name)
    .setFields(fields)

  this._addFilters(query, filters)

  return this.executeSql(query.toString())
}

Database.prototype.delete = function(table, filters) {
  var query = squel.delete()
    .from(table.name)

  this._addFilters(query, filters)

  return this.executeSql(query.toString())
}

Database.prototype.executeSql = function(statement) {
  var resolve
  var promise = new Promise(function(_resolve, _reject) {
    resolve = _resolve
    reject = _reject
  })

  if (DEBUG) { console.log(statement) }
  this.db.transaction(function(tx) {
    tx.executeSql(statement, [], function(tx, results) {
      var result = []

      //result is SQLResultSetRowList object type and has no forEach fn
      for (var i = 0; i < results.rows.length; i++) {
        result.push(results.rows.item(i))
      }

      if (DEBUG) { console.log(result) }

      return resolve(result)
    })
  }, function(error) {
    return reject(new Error(error.message))
  })

  return promise
}

module.exports = Database
