var database = require('../../../database')
var ITEM_TABLE = require('../../../tables').item
var uuidGenerator = require('../../../lib/uuid-generator')
var DRAFT_STATUS = require('../../../lib/status/provider').draft
var EDITED_STATUS = require('../../../lib/status/provider').edited
var footerButtons = require('./footer-buttons')
var eanScanner = require('../../../lib/ean-scanner')
var FormFields = require('./form-fields')
var apiSyncer = require('../../../api/sync')
var tableKeys = require('../../../lib/tables/keys')(ITEM_TABLE)
var alert = require('../../../lib/alert')
var i18n = require('../../../i18n')
var barcodeFinder = require('../../../api/barcode-finder')
var Promise = require('bluebird')
var swal = require('sweetalert2')

var CatalogItem = function(scope, state) {
  var self = this
  var formFields = new FormFields()

  this.state = state
  this.eanField = formFields.getEanField()
  this.nameField = formFields.getNameField()

  scope.onSubmit = this.onFormSubmit.bind(this)
  scope.subButtons = footerButtons({
    fetch: this.onFetch.bind(this),
    scan: this.onEanScan.bind(this)
  }).subButtons

  this.scope = scope

  this._loadSubmitButtonLabel(this.scope.itemId)

  this._loadItem()
  .then(function(values) {
    scope.fields = formFields.loadFields(values[0])
    self._render()
  })
  .catch(function(error) {
    console.error(error)
  })
}

CatalogItem.prototype._loadItem = function() {
  if (!this.scope.itemId) {
    this.scope.title = i18n.t('TITLE_ADD_ITEM')
    return Promise.resolve([])
  }
  this.scope.title = i18n.t('TITLE_EDIT_ITEM')
  return database.read(ITEM_TABLE, tableKeys, { id: this.scope.itemId })
}

CatalogItem.prototype._onCreate = function(formValues) {
  var self = this
  formValues.id = uuidGenerator()
  formValues.status = DRAFT_STATUS

  return database.create(ITEM_TABLE, formValues)
  .then(function() {
      self._syncTable()
    })
}

CatalogItem.prototype._onEdit = function(formValues) {
  var self = this
  formValues.status = EDITED_STATUS

  return database.update(ITEM_TABLE, formValues, { id: this.scope.itemId })
    .then(function() {
      self._syncTable()
    })
}

CatalogItem.prototype._syncTable = function() {
  var self = this
  apiSyncer.syncTable(ITEM_TABLE)
  .then(function() {
    alert.success()
    self.state.go('catalogList')
  })
  .catch(function(error) {
    alert.error()
    console.error(error)
  })
}

CatalogItem.prototype.onFormSubmit = function(formValues) {
  if (this.scope.itemId)
    return this._onEdit(formValues)
  return this._onCreate(formValues)
}

CatalogItem.prototype.onEanScan = function() {
  var self = this
  return eanScanner.activate()
  .then(function(result) { self.eanField.setValue(result.text) })
  .then(function() { self._render() })
}

CatalogItem.prototype.onFetch = function() {
  var self = this
  var eanFieldValue = this.eanField.getValue()

  if (!eanFieldValue) return alert.error('Error', i18n.t('ERROR_NO_EAN'))

  swal({
    text: i18n.t('MSG_FETCH_DATA'),
    showCancelButton: true,
    showLoaderOnConfirm: true,
    type: 'warning',
    preConfirm: function() {
      return barcodeFinder(eanFieldValue)
        .then(function(data) { return Promise.resolve(data) })
        .catch(function(error) { return Promise.reject(error) })
    }
  })
  .then(function(data) { return self._handleApiResponse(data) })
  .catch(function(error) {
    if (error === 'cancel') return
    return Promise.reject(error)
  })
}

CatalogItem.prototype._handleApiResponse = function(data) {
  if (!data.length) {
    return alert.error('', i18n.t('MSG_FETCH_DATA_EMPTY'))
  }
  this.nameField.setValue(data[0].Name)
  this._render()
  return alert.success()
}

CatalogItem.prototype._loadSubmitButtonLabel = function(itemId) {
  var label = itemId ? i18n.t('BUTTON_EDIT') : i18n.t('BUTTON_CREATE')
  this.scope.submitLabel = label
}

CatalogItem.prototype._render = function() {
  if (!this.scope.$$phase) {
    this.scope.$digest()
  }
}

module.exports = CatalogItem
